@@ -1,3 +1,9 @@
 from .movielens100k_data import MovieLens100kDataManager
 from .movielens1M_data import MovieLens1MDataManager
-from .movielens10M_data import MovieLens10MDataManager
\ No newline at end of file
+from .movielens10M_data import MovieLens10MDataManager
+
+__all__ = [
+    "MovieLens100kDataManager",
+    "MovieLens1MDataManager",
+    "MovieLens10MDataManager",
+]
